#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab

import os
import sys
import logging
from oam_common import oam_app_name, oam_endpoint_port
from poaupdater import uAction, uConfig, openapi, uLogging

default_error_action = uAction.get_default_error_action()


class OamAps:
    def __init__(self):
        config = uConfig.Config()
        # Configure logging
        uLogging.init(config)
        logging.basicConfig(level=logging.INFO)
        logging.getLogger().addHandler(uLogging.Handler())
        # Configure OpeApi
        openapi.initFromEnv(config)
        self.api = openapi.OpenAPI()
        # Set app settings
        self.app_name = oam_app_name
        self.endpoint_port = oam_endpoint_port
        self.app_path = self._get_package_file_path('%s-aps.zip' % self.app_name)

    def install(self):
        logging.info('Installing APS package "%s"...' % self.app_name)
        # Import OAM aps package
        logging.debug('Importing aps package...')
        app_info = self._send_api_request_sync('pem.APS.importPackage', package_url='file://%s' % self.app_path)
        # Check is app instance exist
        logging.debug('Checking is app instance exist...')
        app_instance = self._send_api_request_sync('pem.APS.getApplicationInstances', app_id=app_info['application_id'])
        if not app_instance:
            # Provide OAM application instance
            logging.debug('Providing app instance...')
            self._send_api_request_sync('pem.APS.provideApplicationInstance',
                                        subscription_id=0,
                                        app_id=app_info['application_id'],
                                        package_version=app_info["package_version"],
                                        url_path='https://127.0.0.1:%s/%s' % (self.endpoint_port, self.app_name),
                                        rt_id=0)
        logging.info('The APS package was successfully installed.')

    def upgrade(self):
        logging.info('Upgrading APS package "%s"...' % self.app_name)
        # Import new version of OAM package
        logging.debug('Importing aps package...')
        app_info = self._send_api_request_sync('pem.APS.importPackage', package_url='file://%s' % self.app_path)
        package_version = app_info['package_version']

        logging.debug('Upgrading application instance...')
        # Upgrade application instances
        applications = self._send_api_request_sync('pem.APS.getApplications',
                                                   aps_application_id='http://www.odin.com/oam/')
        for app in applications:
            app_instances = self._send_api_request_sync('pem.APS.getApplicationInstances', app_id=app['application_id'])
            for instance in app_instances:
                app_instance_id = instance['application_instance_id']
                root_service_id = instance['root_service_instance_id']
                aps_uid = instance['application_resource_id']
                logging.info('Upgrading application: instance=%d, root_service_id=%d, aps_uid=%s' % (
                    app_instance_id, root_service_id, aps_uid))
                self._send_api_request_sync('pem.APS.upgradeApplicationInstance',
                                            application_instance_id=app_instance_id,
                                            package_version=package_version)

        logging.info('The APS package was successfully upgraded.')

    @staticmethod
    def _get_package_file_path(file_name):
        package_path = os.path.dirname(os.path.abspath(os.path.join(__file__, '..')))
        file_path = os.path.join(package_path, file_name)
        # Checking if file is exist
        if not os.path.isfile(file_path):
            raise Exception('%s file not found.' % file_path)
        return file_path

    def _send_api_request_sync(self, method, **request_params):
        request = self.api.beginRequest()
        method_result = getattr(self.api, method)(**request_params)
        self.api.commit()
        uAction.set_default_error_action("abort")
        openapi.waitRequestComplete(request, method, 1)
        uAction.set_default_error_action(default_error_action)
        return method_result


def main():
    if len(sys.argv) != 2 or sys.argv[1] not in ['--install', '--upgrade']:
        sys.stderr.write("Unknown command '%s'. \n"
                         "Usage: %s --install|--upgrade" % (sys.argv[1], os.path.basename(__file__)))
        return 1

    oam_aps = OamAps()
    if sys.argv[1] == '--install':
        oam_aps.install()
    elif sys.argv[1] == '--upgrade':
        oam_aps.upgrade()


if __name__ == "__main__":
    exit(main())
