import os
import tempfile
import zipfile
import shutil
import sys
import logging
from distutils import dir_util
from oam_common import oam_progect_folder, oam_app_name, log_file
from poaupdater import uConfig, uLogging


class OamEndpoint:
    def __init__(self):
        self.project_folder = oam_progect_folder
        self.app_name = oam_app_name
        self.aps_package_path = self._get_package_file_path('%s-aps.zip' % self.app_name)
        self.log_file = log_file
        self.endpoint_path = os.path.join(self.project_folder, 'www', self.app_name)

    def install(self):
        logging.info('Deploying an endpoint for "%s"...' % self.app_name)
        self._create_endpoint_folder()
        self._create_php_wrapper()
        self._unzip_aps_package()
        logging.info('The endpoint was successfully deployed.')

    def upgrade(self):
        logging.info('Upgrading an endpoint for "%s"...' % self.app_name)
        self._unzip_aps_package()
        logging.info('The endpoint was successfully upgraded.')

    def _create_endpoint_folder(self):
        return self._create_directory(self.endpoint_path)

    def _create_php_wrapper(self):
        folder_path = os.path.join(self.project_folder, 'php-bin')
        self._create_directory(folder_path)
        wrapper_path = os.path.join(folder_path, 'php')

        with open(wrapper_path, mode='w') as wrapper_f:
            wrapper_f.writelines([
                '#!/bin/sh\n',
                '\n',
                'php-cgi "$@"\n'
            ])

        os.chmod(wrapper_path, 0o755)

    def _unzip_aps_package(self):
        temp_dir = self._create_directory(os.path.join(tempfile.gettempdir(), 'oam-endpoint'), 0o700)
        z_file = zipfile.ZipFile(self.aps_package_path)
        for name in z_file.namelist():
            path = None
            if name.startswith('scripts/'):
                path = z_file.extract(name, temp_dir)
            elif name.startswith('library/') or name.startswith('vendors/') or name == 'version':
                path = z_file.extract(name, os.path.realpath(os.path.join(self.endpoint_path, '..')))

            if path:
                if os.path.isdir(path):
                    os.chmod(path, 0o755)
                else:
                    os.chmod(path, 0o644)

        dir_util.copy_tree(os.path.join(temp_dir, 'scripts'), self.endpoint_path)
        shutil.rmtree(temp_dir)

    @staticmethod
    def _create_directory(path, mode=0o755):
        if os.path.exists(path):
            if os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)

        os.makedirs(path, mode)
        return path

    @staticmethod
    def _get_package_file_path(file_name):
        package_path = os.path.dirname(os.path.abspath(os.path.join(__file__, '..')))
        file_path = os.path.join(package_path, file_name)
        # Checking if file is exist
        if not os.path.isfile(file_path):
            raise Exception('%s file not found.' % file_path)
        return file_path


def init():
    config = uConfig.Config()
    uLogging.init(config)
    logging.basicConfig(level=logging.INFO)
    logging.getLogger().addHandler(uLogging.Handler())


def update_locales(endpoint_path):
     messages_path = os.path.realpath(os.path.join(endpoint_path, 'messages/'))
     for file in os.listdir(messages_path):
         if file.endswith('.po'):
             input_file = os.path.join(messages_path, file)
             output_file = os.path.join(messages_path, file.replace('.po','.mo'))
             os.system('msgfmt --output-file %s %s' % (output_file, input_file))


def main():
    if len(sys.argv) != 2 or sys.argv[1] not in ['--install', '--upgrade']:
        sys.stderr.write("Unknown command '%s'. \n"
                         "Usage: %s --install|--upgrade" % (sys.argv[1], os.path.basename(__file__)))
        return 1
    init()
    oam_endpoint = OamEndpoint()
    if sys.argv[1] == '--install':
        oam_endpoint.install()
    elif sys.argv[1] == '--upgrade':
        oam_endpoint.upgrade()

    update_locales(oam_endpoint.endpoint_path)


if __name__ == "__main__":
    exit(main())
