#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab

import os
import sys
import subprocess
import logging
from pwd import getpwnam
from grp import getgrnam
from poaupdater import uConfig, uLogging


def ensure_group_exists(group):
    try:
        logging.info("Checking whether the \"%s\" group exists..." % group)
        getgrnam(group)
        logging.info("OK")
    except KeyError:
        try:
            logging.info("The group \"%s\" was not found. Creating group..." % group)
            cmd = ['groupadd', '-f', group]
            returncode = subprocess.call(cmd, stderr=sys.stdout)
            if returncode != 0:
                raise Exception(
                    "Unable to add the group \"%s\". The following error occurred: code=%d." % (
                        group, returncode))

            logging.info("OK")
        except EnvironmentError, ex:
            raise Exception(
                "Unable to add the group \"%s\". The following error occurred: code=%d, message:%s." % (
                    group, ex.errno, ex.message))


def ensure_user_exists(user, home_dir, group, additional_groups=None):
    try:
        logging.info("Checking whether the \"%s\" user exists..." % user)
        getpwnam(user)
        logging.info("OK")
    except KeyError:
        try:
            logging.info("The user \"%s\" was not found. Creating an account..." % user)
            cmd = ['useradd',
                   '--gid', group,
                   '--create-home',
                   '--home', home_dir,
                   '--shell', '/sbin/nologin',
                   user]
            if additional_groups is not None:
                cmd[3:3] = ['--groups', ','.join(additional_groups)]

            returncode = subprocess.call(cmd, stderr=sys.stdout)
            if returncode != 0:
                raise Exception(
                    "Unable to add the user \"%s\". The following error occurred: code=%d." % (
                        user, returncode))
            # disabling user expiredate
            cmd = ['usermod',
               '--inactive', "-1",
               user]
            returncode = subprocess.call(cmd, stderr=sys.stdout)
            if returncode != 0:
                raise Exception(
                    "Unable to set inactive parameter for user \"%s\". The following error occurred: code=%d." % (
                        user, returncode))
            logging.info("OK")
        except EnvironmentError, ex:
            raise Exception(
                "Unable to add the user \"%s\". The following error occurred: code=%d, message:%s." % (
                    user, ex.errno, ex.message))

    if additional_groups is not None:
        for group in additional_groups:
            logging.info("Checking whether the user \"%s\" is in the group \"%s\"..." % (user, group))
            if user not in getgrnam(group).gr_mem:
                raise Exception("The user \"%s\" must be a member of the group \"%s\"." % (user, group))
            logging.info("OK")


def main():
    # Init log framework.
    config = uConfig.Config()
    uLogging.init(config)
    logging.basicConfig(level=logging.INFO)
    logging.getLogger().addHandler(uLogging.Handler())

    # Parse args.
    if len(sys.argv) < 3:
        sys.stderr.write("Not enough actual parameters. Example: %s user group home_dir" % os.path.basename(__file__))
        exit(1)
    user = sys.argv[1]
    #sys_group = sys.argv[]
    home_dir = sys.argv[2]

    ensure_group_exists(user)
    ensure_user_exists(user=user, home_dir=home_dir, group=user)


if __name__ == "__main__":
    main()
